/*
    Library: libgaudio - multichannel-mixing audio playback for games

    The contents of this file are subject to the Mozilla Public
    License Version 1.1 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/
    
    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.
   
    The Original Code is the libgaudio sound mixing library.
    
    The Initial Developer of the Original Code is A. Schiffler.  
    Portions created by A. Schiffler are Copyright (C) 1999-2000
    A. Schiffler.  All Rights Reserved.
    
    Contributor(s): A. Schiffler <aschiffler@home.com>
                                                                                                                dnl 
    Alternatively, the contents of this file may be used under the
    terms of the GNU General Public License Version 2 or later (the
    "GPL"), in which case the provisions of the GPL are applicable 
    instead of those above.  If you wish to allow use of your 
    version of this file only under the terms of the GPL and not to
    allow others to use your version of this file under the MPL,
    indicate your decision by deleting the provisions above and
    replace them with the notice and other provisions required by
    the GPL.  If you do not delete the provisions above, a recipient
    may use your version of this file under either the MPL or the
    GPL.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    To contact the author try:
    |  Andreas Schiffler                    aschiffler@home.com  |
    |  Senior Systems Engineer  -  Deskplayer.Com Inc., Buffalo  |
    |  4707 Eastwood Cres., Niagara Falls, Ont  L2E 1B4, Canada  |
    |  +1-905-371-3652 (private)  -  +1-905-371-8834 (work/fax)  |

*/

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

#include "gaudio.h"

#define MIN(a,b)    (((a) < (b)) ? (a) : (b))
#define MAX(a,b)    (((a) > (b)) ? (a) : (b))

/* Locks/Unlocks the buffer mutex */
#define LOCK_BUFFERS(ga)    pthread_mutex_lock(&ga->buffer_mutex)
#define UNLOCK_BUFFERS(ga)  pthread_mutex_unlock(&ga->buffer_mutex)

/* Locks/Unlocks the buffer mutex if not paused */
#define LOCK_BUFFERS_NP(ga)    if (!audioPaused(ga)) LOCK_BUFFERS(ga)
#define UNLOCK_BUFFERS_NP(ga)  if (!audioPaused(ga)) UNLOCK_BUFFERS(ga)

/*
    Worker-thread that plays the audio
*/

void* theAudioThread( struct gameaudio* ga )
{
    short playbuffer[ AUDIO_BUFFER_SIZE / sizeof(short) ];
    short zerobuffer[ AUDIO_BUFFER_SIZE / sizeof(short) ];
    struct ga_playing* curplaying;
    short leftsample, rightsample;
    short* curposleft;
    short* curposright;
    int i, j;
    int la, rb, lb, ra, larb, lbra;

    /* Build clearbuffer */
    for( i = 0; i < (AUDIO_BUFFER_SIZE/sizeof(short)); i++ )
        zerobuffer[i] = 0;

    /* Loop ... playing sound */
    while( 1 )
    {
        LOCK_BUFFERS(ga);

        if( ga->background_stream == NULL )
        {
            /* Clear to zero */
            memcpy( playbuffer,zerobuffer,AUDIO_BUFFER_SIZE );
        }
        else
        {
            /* Preset from background stream */
            if( fread(playbuffer, 1, AUDIO_BUFFER_SIZE, ga->background_stream)
                < AUDIO_BUFFER_SIZE )
            {
                /* Close */
                pclose( ga->background_stream );
                ga->background_stream = NULL;
                ga->background_stream_bytes = 0;
                /* Clear to zero after all */
                memcpy( playbuffer, zerobuffer, AUDIO_BUFFER_SIZE );
            } else {
            	/* Store stream bytes */
            	ga->background_stream_bytes += AUDIO_BUFFER_SIZE;
            }
        }

        /* Needs mixing? */
        if( ga->num_playing > 0 )
        {
            /* Create playbuffer */
            curposleft  = playbuffer;
            curposright = playbuffer;
            curposright++;

            for( i = 0; i < (AUDIO_BUFFER_SIZE/4); i++ )
            {
                curplaying = ga->playing;
                for( j = 0; j < ga->num_playing; j++ )
                {
                    /* Get source samples and advance sample positions */
                    leftsample=*curplaying->curpos;
                    (short*) curplaying->curpos++;
                    rightsample=*curplaying->curpos;
                    (short*) curplaying->curpos++;

                    /* Mangle left and right components with the  */
                    /* pan/volume coefficients                    */
                    la = (int)leftsample  * (curplaying->a);
                    rb = (int)rightsample * (curplaying->b);
                    lb = (int)leftsample  * (curplaying->b);
                    ra = (int)rightsample * (curplaying->a);
                    if (la<0) {
                     larb = MAX(la+rb,(int)(-32768*256));
                    } else {
                     larb = MIN(la+rb,(int)(32767*256));
                    }
                    if (la<0) {
                     lbra = MAX(lb+ra,(int)(-32768*256));
                    } else {
                     lbra = MIN(lb+ra,(int)(32767*256));
                    }
                    leftsample  = (short)(larb/256);
                    rightsample = (short)(lbra/256);
                    
                    /* Mix-in current sample - left */
                    if( leftsample < 0 )
                    {
                        *curposleft = MAX((int)*curposleft +
                                      (int)leftsample, (int)-32768);
                    }
                    else
                    {
                        *curposleft = MIN((int)*curposleft +
                                      (int)leftsample, (int)32767);
                    }

                    /* Mix-in current sample - right */
                    if( rightsample < 0 )
                    {
                        *curposright = MAX((int)*curposright +
                                       (int)rightsample, (int)-32768);
                    } else {
                        *curposright = MIN((int)*curposright +
                                       (int)rightsample, (int)32767);
                    }

                    /* Check if we are at the end */
                    if( curplaying->curpos == curplaying->endpos )
                    {
                        /* Update repeat counter */
                        curplaying->repeat_count--;
                        if( curplaying->repeat_count < 0 )
                        {
                            /* Looped sample - reset position and counter */
                            curplaying->repeat_count = 0;
                            curplaying->curpos = curplaying->rootpos;
                        }
                        else if( curplaying->repeat_count > 0 )
                        {
                            /* Multi-play sample - reset position */
                            curplaying->curpos = curplaying->rootpos;
                        }
                        else
                        {
                            /* Done - delete entry from list */
                            if( j != (ga->num_playing - 1) )
                            {
                                ga->playing[j] = ga->playing[ga->num_playing-1];
                            }

                            ga->num_playing--;

                            /* Check entry again */
                            curplaying--;
                            j--;
                        }
                    }

                    /* Next in playlist */
                    curplaying++;
                }

                /* Advance pointers */
                curposleft += 2;
                curposright += 2;
            }
        }

        UNLOCK_BUFFERS(ga);

        /* Write playbuffer to audio device */
        write( ga->audio_fd, playbuffer, AUDIO_BUFFER_SIZE );
    }
}


/*
    Start gameaudio system: open device (usually /dev/dsp), setup device
    parameters, initialize tables, start thread.  Returns -1 on error.
    stopAudio does not need to be called if startAudio does not succeed.
*/

int startAudio( struct gameaudio* ga, const char* device_name )
{
    int format, stereo, speed, fragsize;

    /* Init variables */
    ga->paused      = 0;
    ga->num_samples = 0;
    ga->num_playing = 0;
    ga->playing_count = 0;
    ga->background_stream = NULL;
    ga->background_stream_bytes = 0;
    
    /* Setup audio device */
    if( (ga->audio_fd = open( device_name, O_WRONLY, 0 )) == -1 )
    {
        /* Opening device failed */  
        perror( device_name );
        return( -1 );
    }

    /* Setup device parameters */
    format   = AFMT_S16_LE;
    stereo   = 1;
    speed    = GAUDIO_RATE;
    fragsize = 0x0004000c;

    /* Program audio device with parameters */
    if( ioctl(ga->audio_fd, SNDCTL_DSP_SETFRAGMENT, &fragsize) == -1 )
    {
        perror("SNDCTL_DSP_SETFRAGMENT");  
        goto err;
    }
    if( ioctl(ga->audio_fd, SNDCTL_DSP_SETFMT, &format) == -1 )
    {
        perror("SNDCTL_DSP_SETFMT");  
        goto err;
    }
    if( ioctl(ga->audio_fd, SNDCTL_DSP_STEREO, &stereo) == -1 )
    {
        perror("SNDCTL_DSP_STEREO");  
        goto err;
    }
    if( ioctl(ga->audio_fd, SNDCTL_DSP_SPEED, &speed) == -1 )
    {
        perror("SNDCTL_DSP_SPEED");  
        goto err;
    }

    /* Init mutex */ 
    if( pthread_mutex_init( &ga->buffer_mutex, NULL ) == -1 )
    {
        perror( "pthread_mutex_init()" );
        goto err;
    }

    /* Start playing thread */
    if( pthread_create( &ga->audio_thread, NULL, (void*) theAudioThread,
                        (void*) ga) == -1 )
    {
        perror( "pthread_create()" );
        goto err;
    }

    return( 0 );
    
err:

    close( ga->audio_fd );
    ga->audio_fd = -1;
    return(-1);  
}


/*
   Stop gameaudio system: turn off all playing sounds, kill thread, close
   device.  Returns -1 on error.
*/

int stopAudio( struct gameaudio* ga )
{
    int ret;
    int i;

    if( ga->audio_fd == -1 )
        return( -1 );

    ret = 0;

    /* Stop grabbing thread */
    if( pthread_cancel( ga->audio_thread ) == -1 )
    {
        perror( "pthread_cancel()" );
        ret = -1;
    }

    /* Unload samples from memory */
    for( i = 0; i < ga->num_samples; i++ )
    {
        free( ga->samples[i].data );
    }
    ga->num_samples = 0;

    /* Close device */
    close( ga->audio_fd );
    ga->audio_fd = -1;

    return( ret );
}


/*
    Pauses audio output until unpauseAudio() is called.

    This may safely be called mulitple times.
    While audio is paused calls to playSample and playBackground are ignored.
    stopSample, stopBackground, and checkBackground work as expected.

    Do not call loadSample or copySample when audio is paused.
*/

void pauseAudio( struct gameaudio* ga )
{
    if( audioValid( ga ) && ga->paused == 0 )
    {
        LOCK_BUFFERS(ga);
        ga->paused = 1;
    }
}


/*
   Continues paused audio.
   Does nothing if pauseAudio() has not been called.
*/

void unpauseAudio( struct gameaudio* ga )
{
    if( audioValid( ga ) && ga->paused )
    {
        ga->paused = 0;
        UNLOCK_BUFFERS(ga);
    }
}


/*
    Load a (signed 16bit stereo 44100Hz) sample into memory for use with the
    gameaudio system.  Blanks one or the other channel depending on mode.
    Returns sample id or -1 on error.
*/

#define LOAD_STEREO	0
#define LOAD_LEFT	1
#define LOAD_RIGHT	2

/* Generic load function */

int loadSampleStereo( struct gameaudio* ga, const char* filename, int mode) 
{
    SF_INFO infile_info;
    SNDFILE *infile;
    struct ga_sample* sample;
    int id;
    int read_length;
    int i;
    unsigned char *cursample;

    if( ! audioValid( ga ) )
        return( -1 );

    /* Check entry count */
    if( ga->num_samples >= MAX_NUM_SAMPLES )
    {
        fprintf( stderr, "gaudio: sample list is full\n" );
        return( -1 );
    }

    id     = ga->num_samples;
    sample = &ga->samples[ id ];

    ga->num_samples++;

    /* Open input file */
    if (!(infile = sf_open_read(filename, &infile_info))) {
        fprintf( stderr, "gaudio: error opening sample %s\n", filename );
        return( -1 );
    }

    /* Set info for RAW files */
    if (infile_info.format == SF_FORMAT_RAW) {
    	    infile_info.channels=2;
    	    infile_info.samplerate=44100;
    	    infile_info.pcmbitwidth=16;
	    /* Print info */
  	    fprintf (stderr, "Input file %s: %i RAW samples - assuming %i channels, %i rate, %i bits\n",filename, infile_info.samples, infile_info.channels, infile_info.samplerate, infile_info.pcmbitwidth);
    } else {
	    /* Print info */
  	    fprintf (stderr, "Input file %s: %i samples, %i channels, %i rate, %i bits\n",filename, infile_info.samples, infile_info.channels, infile_info.samplerate, infile_info.pcmbitwidth);
    }
    
    /* Lock buffers while loading sample */
    LOCK_BUFFERS(ga);

    /* Setup sample length */
    sample->numsamples = (AUDIO_BUFFER_SIZE/BYTES_PER_SAMPLE) *
            ((infile_info.samples/(AUDIO_BUFFER_SIZE/BYTES_PER_SAMPLE)+1));

    /* Create buffer */
    sample->data = malloc( BYTES_PER_SAMPLE * sample->numsamples );
    if( sample->data == NULL )
    {
        fprintf( stderr, "gaudio: error allocating sample memory\n" );
        ga->num_samples--;
        UNLOCK_BUFFERS(ga);
        return( -1 );
    }

    /* Fill with neutral value */
    memset( sample->data, 0, BYTES_PER_SAMPLE * sample->numsamples );

    /* Load file into buffer */
    read_length = sf_read_short (infile, (void*) sample->data, infile_info.samples*infile_info.channels);

    /* Clone channel for mono samples */
    if (infile_info.channels==1) {
     for (i=infile_info.samples-1; i>0; i--) {
      sample->data[2*i]=sample->data[i]; 
      sample->data[2*i+1]=sample->data[i]; 
     }
    }

    /* Rescale for 8bit samples */
    if (infile_info.pcmbitwidth==8) {
     for (i=0; i<sample->numsamples; i++) {
      (unsigned short)sample->data[2*i] *= 256;
      (unsigned short)sample->data[2*i+1] *= 256;
     }
    }
    
    /* Blank one or the other channel depending on mode */
    if (mode == LOAD_LEFT) {
    	 cursample=(unsigned char *)sample->data;
    	 if (BYTES_PER_SAMPLE==2) {
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 2;
    	  }
    	 } else if (BYTES_PER_SAMPLE==4) {
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 1;
    	   *cursample=0;
    	   cursample += 3;
    	  }
    	 }
    } else if (mode == LOAD_RIGHT) {
    	 cursample=(unsigned char *)sample->data;
    	 if (BYTES_PER_SAMPLE==2) {
    	 cursample++;
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 2;
    	  }
    	 } else if (BYTES_PER_SAMPLE==4) {
     	  cursample += 2;
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 1;
    	   *cursample=0;
    	   cursample += 3;
    	  }
    	 }
    }

    /* Calculate end-of-sample position pointer */ 
    sample->dataendmarker = sample->data + (2 * sample->numsamples);

    UNLOCK_BUFFERS(ga);

    /* Close file */
    sf_close( infile );

    return( id );
}

/* Exported standard load function */

int loadSample( struct gameaudio* ga, const char* filename ) 
{
 return (loadSampleStereo(ga,filename,LOAD_STEREO));
}


int loadSampleLeft( struct gameaudio* ga, const char* filename ) 
{
 return (loadSampleStereo(ga,filename,LOAD_LEFT));
}

int loadSampleRight( struct gameaudio* ga, const char* filename ) 
{
 return (loadSampleStereo(ga,filename,LOAD_RIGHT));
}

/*
  Creates a sample from data (signed 16bit stereo 44100Hz) loaded by
  external means.  Returns sample id or -1 on error.
*/

/* Internal generic load function */

int copySampleStereo( struct gameaudio* ga, void* data, int bytelen, int mode )
{
    struct ga_sample* sample;
    int id;
    int pad;
    int i;
    unsigned char *cursample;

    if( ! audioValid( ga ) )
        return( -1 );

    /* Check entry count */
    if( ga->num_samples >= MAX_NUM_SAMPLES )
    {
        fprintf( stderr, "gaudio: sample list is full\n" );
        return( -1 );
    }

    id     = ga->num_samples;
    sample = &ga->samples[ id ];

    /* Lock buffers while loading sample */
    LOCK_BUFFERS(ga);

    /* Setup sample length */
    sample->numsamples = (AUDIO_BUFFER_SIZE *
                         ((bytelen/AUDIO_BUFFER_SIZE)+1)) / BYTES_PER_SAMPLE;

    /* Create buffer */
    sample->data = malloc( BYTES_PER_SAMPLE * sample->numsamples );
    if( sample->data )
    {
        memcpy( sample->data, data, bytelen );

        /* Fill buffer padding with neutral value */
        pad = (BYTES_PER_SAMPLE * sample->numsamples) - bytelen;
        if( pad > 0 )
            memset( ((char*) sample->data) + bytelen, 0, pad );

        /* Blank one or the other channel depending on mode */
        if (mode == LOAD_LEFT) {
    	 cursample=(unsigned char *)sample->data;
    	 if (BYTES_PER_SAMPLE==2) {
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 2;
    	  }
    	 } else if (BYTES_PER_SAMPLE==4) {
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 1;
    	   *cursample=0;
    	   cursample += 3;
    	  }
    	 }
        } else if (mode == LOAD_RIGHT) {
    	 cursample=(unsigned char *)sample->data;
    	 if (BYTES_PER_SAMPLE==2) {
    	 cursample++;
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 2;
    	  }
    	 } else if (BYTES_PER_SAMPLE==4) {
     	  cursample += 2;
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 1;
    	   *cursample=0;
    	   cursample += 3;
    	  }
    	 }
        }

        /* Calculate end-of-sample position pointer */ 
        sample->dataendmarker = sample->data + (2 * sample->numsamples);

        ga->num_samples++;
    }
    else
    {
        fprintf( stderr, "gaudio: error allocating sample memory\n" );
        id = -1;
    }

    UNLOCK_BUFFERS(ga);

    return( id );
}

/* Exported standard load function */

int copySample( struct gameaudio* ga, void* data, int bytelen )
{
 return(copySampleStereo(ga,data,bytelen,LOAD_STEREO));
}

int copySampleLeft( struct gameaudio* ga, void* data, int bytelen )
{
 return(copySampleStereo(ga,data,bytelen,LOAD_LEFT));
}

int copySampleRight( struct gameaudio* ga, void* data, int bytelen )
{
 return(copySampleStereo(ga,data,bytelen,LOAD_RIGHT));
}

/*
    Play sample referenced by id 'repeat_count' times. If the repeat_count is
    0 playing will loop forever. Return (negative) playing ID or 0.
*/

int playSample( struct gameaudio* ga, int sample_id, int repeat_count ) 
{
    int playing_id=0;
    struct ga_playing* curplaying;
    struct ga_sample* cursample;

    if( audioPaused( ga ) )
        return(0);

    /* Sanity check */
    if( (sample_id < 0) || (sample_id >= ga->num_samples) )
        return(0);

    LOCK_BUFFERS(ga);

    /* Check if there is a slot free */
    if( ga->num_playing < MAX_NUM_PLAYING )
    {
    	/* Update playing count (count down, wrap around without uniqueness check) */
    	ga->playing_count--;
    	if (ga->playing_count>0) ga->playing_count=-1;
        playing_id=ga->playing_count;
    	
        /* Fill slot with info */
        cursample  = &ga->samples[ sample_id ];
        curplaying = &ga->playing[ ga->num_playing ];

        curplaying->rootpos      = cursample->data;
        curplaying->curpos       = cursample->data;
        curplaying->endpos       = cursample->dataendmarker;
        curplaying->repeat_count = repeat_count;
        curplaying->sample_id    = sample_id;
        curplaying->playing_id   = ga->playing_count;
	curplaying->a		 = 0;
	curplaying->b 		 = 256;
	
        ga->num_playing++;        
    }

    UNLOCK_BUFFERS(ga);
    
    return (playing_id);
}

/* 
    Set volume and panning parameters for a sample
*/

void ctrlSample( struct gameaudio* ga, int sample_id, int volume, int pan)
{
    int i;
    struct ga_playing* curplaying;
    int a,b;

    /* Sanity check on id */
    if( (sample_id < 0) || (sample_id >= ga->num_samples) )
        return;

    /* Clip parameters */
    if (volume<0) {
     volume=0;
    } else if (volume>256) {
     volume=256;
    }
    if (pan<0) {
     pan=0;
    } else if (pan>256) {
     pan=256;
    }
     
    /* Calculate volume and pan parameters */
    a=(volume*(256-pan))/256;
    b=(volume*(pan))/256;

    LOCK_BUFFERS_NP(ga);

    /* Loop through playlist and set parameters for entries that match the id */
    curplaying = ga->playing;

    if (sample_id<0) {

     for( i = 0; i < ga->num_playing; i++ )
     {
        if( curplaying->playing_id == sample_id )
        {
            /* Set parameters */
            curplaying->a=a;
            curplaying->b=b;
        }
        else
        {
            /* Next in playlist */
            curplaying++;
        }
     }

    } else {
    
     for( i = 0; i < ga->num_playing; i++ )
     {
        if( curplaying->sample_id == sample_id )
        {
            /* Set parameters */
            curplaying->a=a;
            curplaying->b=b;
        }
        else
        {
            /* Next in playlist */
            curplaying++;
        }
     }

    }
    
    UNLOCK_BUFFERS_NP(ga);
}

/*
    Stop all playing samples with the given id.
*/

void stopSample( struct gameaudio* ga, int sample_id )
{
    int i;
    struct ga_playing* curplaying;

    /* Sanity check on id */
    if( (sample_id < 0) || (sample_id >= ga->num_samples) )
        return;

    LOCK_BUFFERS_NP(ga);

    /* Loop through playlist and remove all entries that match id */
    curplaying = ga->playing;

    if (sample_id<0) {
     for( i = 0; i < ga->num_playing; i++ )
     {
        if( curplaying->playing_id == sample_id )
        {
            /* Done - delete from list */
            if( i != (ga->num_playing-1) )
            {
                ga->playing[ i ] = ga->playing[ ga->num_playing-1 ];
            }
            ga->num_playing--;
            /* Check entry again */
            i--;
        }
        else
        {
            /* Next in playlist */
            curplaying++;
        }
     }
    } else {
     for( i = 0; i < ga->num_playing; i++ )
     {
        if( curplaying->sample_id == sample_id )
        {
            /* Done - delete from list */
            if( i != (ga->num_playing-1) )
            {
                ga->playing[ i ] = ga->playing[ ga->num_playing-1 ];
            }
            ga->num_playing--;
            /* Check entry again */
            i--;
        }
        else
        {
            /* Next in playlist */
            curplaying++;
        }
     }
    }
    
    UNLOCK_BUFFERS_NP(ga);
}


/*
    Stop all playing samples.  Any background audio continues to play.
*/

void stopAllSamples( struct gameaudio* ga )
{
    LOCK_BUFFERS_NP(ga);

    ga->num_playing = 0;

    UNLOCK_BUFFERS_NP(ga);
}


/*
    Start background playing of a MP3 file given in filename.
    Any currently playing background audio will be stopped.
*/

void playBackground( struct gameaudio* ga, const char* filename )
{
    char commandname[ FILENAME_MAX + 60 ];

    if( ! audioValid( ga ) || audioPaused( ga ) )
        return;

    LOCK_BUFFERS(ga);

    sprintf( commandname, "/usr/bin/mpg123 -s -q -r %d --stereo %s",
             GAUDIO_RATE, filename );

    if( ga->background_stream != NULL )
        pclose( ga->background_stream );

    ga->background_stream_bytes=0;
    
    if( (ga->background_stream = popen(commandname,"r")) == NULL )
    {
        fprintf( stderr, "gaudio: popen() error\n" );
    }

    UNLOCK_BUFFERS(ga);
}


/*
    Stop currently playing MP3 stream.
*/

void stopBackground( struct gameaudio* ga )
{
    if( audioValid( ga ) )
    {
        LOCK_BUFFERS_NP(ga);

        if( ga->background_stream != NULL )
        {
            pclose( ga->background_stream );
            ga->background_stream = NULL;
            ga->background_stream_bytes = 0;
        }

        UNLOCK_BUFFERS_NP(ga);
    }
}


/*
   Check if the background MP3 stream is playing (1=playing, 0=not playing)
*/

int checkBackground( struct gameaudio* ga )
{
    int retval = 0;

    if( audioValid( ga ) )
    {
        LOCK_BUFFERS_NP(ga);

        retval = (ga->background_stream != NULL);

        UNLOCK_BUFFERS_NP(ga);
    }

    return( retval );
}


/*
   Return the time of background playback (in seconds) or 0.0 if not playing
*/

float timeBackground( struct gameaudio* ga )
{
    float retval = 0.0;

    if( audioValid( ga ) )
    {
        LOCK_BUFFERS_NP(ga);

        if (ga->background_stream != NULL) {
        	retval = (float)(ga->background_stream_bytes)/(float)(GAUDIO_RATE*BYTES_PER_SAMPLE);
        }

        UNLOCK_BUFFERS_NP(ga);
    }

    return( retval );
}


/*EOF*/
